@@ -440,7 +440,7 @@ class DiogenesBarrel extends DiogenesPage
   function script_uri()
   {
     if ($this->barrel->vhost)
-      return preg_replace("/^(.*)\/site\/{$this->alias}\/(.*)/", "/\$2",$_SERVER['REQUEST_URI']);
+      return preg_replace("/^(.*)\/site(\.php)?\/{$this->alias}\/(.*)/", "/\$3",$_SERVER['REQUEST_URI']);
     else
       return $_SERVER['REQUEST_URI'];
   }
