@@ -219,7 +219,7 @@ class DiogenesPage extends DiogenesCorePage
    * @param data
    */
   function log($action,$data="") {
-    if ($this->checkLogger())
+    if (isset($_SESSION['log']) && is_object($_SESSION['log']))
       $_SESSION['log']->log($action,$data);
   }
 
