@@ -20,10 +20,10 @@
 
 
 require_once 'diogenes.spool.inc.php';
+require_once 'diogenes.icons.inc.php';
 // dependency on PEAR
 require_once 'System.php';
 
-
 /** This class handles Diogenes RCS operations.
  */
 class DiogenesRcs extends DiogenesSpool {
@@ -83,7 +83,17 @@ class DiogenesRcs extends DiogenesSpool {
     return $this->rcsPath($dir,$file).",v";
   }
 
+  
+  /** Check whether a file is registered in RCS.
+   *
+   * @param dir parent directory
+   * @param file the RCS entry
+   */
+  function checkFile($dir, $file) {
+    return is_file($this->rcsFile($dir, $file));
+  }
 
+  
   /** Perform sanity check on an RCS directory
    *  and the corresponding checkout in the spool
    *
@@ -108,11 +118,15 @@ class DiogenesRcs extends DiogenesSpool {
   {
     $this->info("RCS : checkout out $file ($rev)..");
     $rfile = $this->rcsFile($dir,$file);
-    exec("co -r".escapeshellarg($rev)." ".escapeshellarg($rfile).
-         " ".escapeshellarg("$output/$file"));
+    if ($this->cmdExec("co -r".escapeshellarg($rev)." ".escapeshellarg($rfile)." ".escapeshellarg("$output/$file")))
+    {
+      $this->info("RCS : Error, checkout failed!");
+      $this->info($this->cmdStatus());
+      return;
+    }         
     return "$output/$file";
   }
-
+  
 
   /** Commit an RCS item. Returns true for succes, false for an error.
   *
@@ -123,12 +137,12 @@ class DiogenesRcs extends DiogenesSpool {
   */
   function commit($dir,$file,$content,$message="")
   {
-    $this->info("RCS : checking in $file..");
+    $this->info("RCS : checking in '$file'..");
 
     // check directories
     if (!$this->checkDir($dir)) {
         // error
-        $this->info("Error: RCS sanity check for '$dir' failed!");
+        $this->info("RCS : Error, RCS sanity check for '$dir' failed!");
         return false;
     }
 
@@ -140,35 +154,41 @@ class DiogenesRcs extends DiogenesSpool {
 
     // if the RCS file does not exist, create it
     if (!file_exists($rfile)) {
-      system("rcs -i ".escapeshellarg($rfile),$ret);
-      if ($ret) {
+      if ($this->cmdExec("rcs -i ".escapeshellarg($rfile)))
+      {
         // error
-        $this->info("Error: could not initialise RCS file $rfile!");
+        $this->info("RCS : Error, could not initialise RCS file '$rfile'!");
+        $this->info($this->cmdStatus());
         return false;
       }
     }
 
     // lock the spool file
-    system("co -l ".escapeshellarg($rfile)." ".escapeshellarg($sfile),$ret);
-    if ($ret) {
+    if ($this->cmdExec("co -l ".escapeshellarg($rfile)." ".escapeshellarg($sfile)))
+    {
         // error
-        $this->info("Error: could not get RCS lock on file!");
+        $this->info("RCS : Error, could not get RCS lock on file '$file'!");
+        $this->info($this->cmdStatus());
         return false;
     }
     if ($fp = fopen($sfile,"w")) {
       fwrite($fp,$content);
       fclose($fp);
-      system("ci -w".escapeshellarg($this->login). ($message ? " -m".escapeshellarg($message) : "").
-             " ". escapeshellarg($sfile). " ". escapeshellarg($rfile),$ret);
-      if ($ret) {
+      
+      if ($this->cmdExec("ci -w".escapeshellarg($this->login). ($message ? " -m".escapeshellarg($message) : "").
+             " ". escapeshellarg($sfile). " ". escapeshellarg($rfile)))
+      {
         // error
-        $this->info("Error: checkin failed!");
+        $this->info("RCS : Error, checkin failed!");
+        $this->info($this->cmdStatus());        
         return false;
       }
-      system("co ".escapeshellarg($rfile)." ".escapeshellarg($sfile),$ret);
-      if ($ret) {
+      
+      if ($this->cmdExec("co ".escapeshellarg($rfile)." ".escapeshellarg($sfile)))
+      {
         // error
-        $this->info("Error: checkout after checkin failed!");
+        $this->info("RCS : Error, checkout after checkin failed!");
+        $this->info($this->cmdStatus());        
         return false;
       }
     }
@@ -176,13 +196,46 @@ class DiogenesRcs extends DiogenesSpool {
   }
 
 
+  
+ /** Make a copy of an RCS item to a given location.
+  *
+  * @param sdir the source directory
+  * @param sfile the source RCS entry
+  * @param ddir the destination directory
+  * @param dfile the destination RCS entry
+  */
+  function copy($sdir,$sfile,$ddir,$dfile)
+  {    
+    $this->info("RCS : copying '$sfile' to '$ddir/$dfile'..");
+    
+    $spath = $this->spoolPath($sdir, $sfile);
+    if (!is_file($spath)) {
+      $this->info("Error: source file '$spath' does not exist!");
+      return false;
+    }
+    if (!$this->checkDir($ddir)) 
+    {
+      $this->info("Error: directory '$ddir' does not exist!");
+      return false;
+    }
+    if ($this->checkFile($ddir, $dfile))
+    {
+      $this->info("Error: file '$dfile' already exists in '$ddir'!");
+      return false;    
+    }
+    return $this->commit($ddir,$dfile,
+                        file_get_contents($spath),
+                        "copied from '$ddir/$sfile'");
+  }
+   
+     
   /** Delete an RCS file and its corresponding spool entry.
   *
   * @param dir parent directory
   * @param file the RCS entry
   */
   function del($dir,$file) {
-    $this->info("RCS : deleting $file..");
+    $this->info("RCS : deleting '$file'..");
     $this->log("rcs_delete","{$this->alias}:$dir/$file");
     @unlink($this->spoolPath($dir,$file));
     @unlink($this->rcsFile($dir,$file));
@@ -199,9 +252,9 @@ class DiogenesRcs extends DiogenesSpool {
   function diff($dir,$file,$r1,$r2)
   {
     $rfile = $this->rcsFile($dir,$file);
-    $this->info("RCS : diffing $file ($r1 to $r2)..");
-    return shell_exec("rcsdiff  -r".escapeshellarg($r1).
-      " -r".escapeshellarg($r2)." ".escapeshellarg($rfile));
+    $this->info("RCS : diffing '$file' ($r1 to $r2)..");
+    $this->cmdExec("rcsdiff  -r".escapeshellarg($r1). " -r".escapeshellarg($r2)." ".escapeshellarg($rfile));
+    return $this->cmd_output;
   }
 
 
@@ -214,6 +267,8 @@ class DiogenesRcs extends DiogenesSpool {
    */
   function importWordFile($dir,$htmlfile,$wordfile)
   {
+    global $globals;
+    
     if (!$globals->word_import) {
       $this->info("Error : support for word import is disabled!");
       return false;
@@ -246,11 +301,12 @@ class DiogenesRcs extends DiogenesSpool {
       return false;
     }
 
-    system("wvHtml --targetdir=".escapeshellarg($tmpdir).
+    if ($this->cmdExec("wvHtml --targetdir=".escapeshellarg($tmpdir).
            " --charset=iso-8859-15 ".
-           escapeshellarg($wordfile)." ".escapeshellarg($tmphtmlfile), $ret);
-    if ($ret) {
+           escapeshellarg($wordfile)." ".escapeshellarg($tmphtmlfile)))
+    {
       $this->info("Error : wvHtml returned an error!");
+      $this->info($this->cmdStatus());
       return false;
     }
 
@@ -288,7 +344,8 @@ class DiogenesRcs extends DiogenesSpool {
    */
   function logEntries($dir,$file) {
     $rfile = $this->rcsFile($dir,$file);
-    return shell_exec("rlog ".escapeshellarg($rfile));
+    $this->cmdExec("rlog ".escapeshellarg($rfile));
+    return $this->cmd_output;
   }
 
 
@@ -300,7 +357,7 @@ class DiogenesRcs extends DiogenesSpool {
    function logParse($dir,$file)
    {
     // get the log, drop last 2 lines
-    $lines = explode("\n", $this->logEntries($dir,$file));
+    $lines = $this->logEntries($dir,$file);
     array_pop($lines);
     array_pop($lines);
 
@@ -325,6 +382,71 @@ class DiogenesRcs extends DiogenesSpool {
   }
 
 
+ /** Move an RCS item to a given location.
+  *
+  * @param sdir the source directory
+  * @param sfile the source RCS entry
+  * @param ddir the destination directory
+  * @param dfile the destination RCS entry
+  */
+  function move($sdir,$sfile,$ddir,$dfile)
+  {    
+    $this->info("RCS : moving '$sfile' to '$ddir/$dfile'..");
+    
+    
+    // check source files
+    $spath = $this->spoolPath($sdir, $sfile);
+    $srpath = $this->rcsFile($sdir, $sfile);    
+            
+    if (!is_file($spath)) {
+      $this->info("Error: source file '$spath' does not exist!");
+      return false;
+    }
+    
+    if (!is_file($srpath)) {
+      $this->info("Error: source RCS file '$srpath' does not exist!");
+      return false;
+    }
+    
+    // check destination
+    $dpath = $this->spoolPath($ddir, $dfile);
+    $drpath = $this->rcsFile($ddir, $dfile);    
+    
+    if (!$this->checkDir($ddir)) 
+    {
+      $this->info("Error: directory '$ddir' does not exist!");
+      return false;
+    }
+    
+    if (file_exists($dpath)) {
+      $this->info("Error: file '$dfile' already exists in '$ddir'!");
+      return false;    
+    }
+    
+    if (file_exists($drpath)) {
+      $this->info("Error: file '".basename($drpath)."' already exists in '$ddir'!");
+      return false;    
+    }
+    
+    if (!rename($spath, $dpath))
+    {
+      $this->info("Error: failed to move '".basename($spath)."' to '".basename($dpath)."' in '$ddir'!");
+      return false;        
+    }
+    
+    if (!rename($srpath, $drpath))
+    {
+      $this->info("Error: failed to move '".basename($srpath)."' to '".basename($drpath)."' in '$ddir'!");
+      return false;        
+    }
+
+        
+    //$this->log("rcs_move","{$this->alias}:$dir/$file");    
+    
+    return true;
+  }
+ 
+ 
   /** Add a new RCS-managed directory.
    *
    * @param parent the parent directory
@@ -346,12 +468,10 @@ class DiogenesRcs extends DiogenesSpool {
    */
   function dispDiff($dir,$file,$r1,$r2)
   {
-    $raw = $this->diff($dir,$file,$r1,$r2);
-
     $lns = "[0-9]+|[0-9]+,[0-9]+";
 
-    // strip any leading comments
-    $lines = split("\n",$raw);
+    // get diff, strip any leading comments
+    $lines = $this->diff($dir,$file,$r1,$r2);    
     $line = "";
     while (!preg_match("/^($lns)([acd])($lns)/",$line))
       $line = array_shift($lines);
@@ -397,14 +517,14 @@ class DiogenesRcs extends DiogenesSpool {
    */
   function dispDir($dir,$loc,$canedit) {
     $entries = array();
-    $odd = false;
 
     if ($pdir = @opendir($this->rcsPath($dir))) {
       while ( ($file = readdir($pdir)) !== false) {
         if ( ($file != ".") && ($file != "..") )
         {
-          array_push($entries,$this->dispEntry($dir,$loc,$file,$odd,$canedit));
-          $odd = !$odd;
+          $entry = $this->dispEntry($dir,$loc,$file,$canedit);
+          if (!empty($entry)) 
+            array_push($entries, $entry);
         }
       }
       closedir($pdir);
@@ -418,86 +538,63 @@ class DiogenesRcs extends DiogenesSpool {
   * @param dir parent directory
   * @param loc visible location for parent directory
   * @param file the RCS "item"
-  * @param odd is this an odd row in the table?
   * @param canedit can we edit files this entry?
   */
-  function dispEntry($dir,$loc,$file,$odd,$canedit) {
+  function dispEntry($dir,$loc,$file,$canedit) {
     global $globals;
 
-    $click = $view = $edit = $del = $size = $rev = "";
+    $view = $edit = $del = $size = $rev = "";
 
     $myitem = $this->rcsPath($dir,$file);
-
-    if (is_dir($myitem)) {
+    
+    // check the RCS entry exists
+    if (!file_exists($myitem)) {
+      $this->info("RCS entry '$myitem' does not exist!");
+      return;
+    }
+    
+    if (is_dir($myitem))
+    {
     // this is a directory, this should not happen!
-      $this->kill("Unexpected directory in RCS : '$myitem'");
-    } else if (substr($file,-2) == ",v") {
+      $this->info("Unexpected directory in RCS, skipping : '$myitem'");
+      return;
+    }
+    else if (substr($file,-2) == ",v")
+    {
     // this is an RCS file
       $file = substr($file,0,-2);
-      $url = ($loc ? "$loc/" : "") . $file;
-      $view = $this->caller->urlSite($url);
-      $size = $this->dispSize(filesize($this->spoolPath($dir,$file)));
-      $icon = "/icons/small/text.gif";
+          
+      // check we have a working copy of this item
+      $spoolitem = $this->spoolPath($dir,$file);
+      if (!is_file($spoolitem)) 
+      {
+        $this->info("Could not find working copy '$spoolitem'!");
+        $size = "";
+        $icon = "";
+      } else {      
+        $size = $this->dispSize(filesize($spoolitem));
+        $icon = $globals->icons->get_mime_icon($spoolitem);        
+      }
+            
       // revision info
       $myrev = array_shift($this->logParse($dir,$file));
-      $rev = "?action=revs&amp;dir=$dir&amp;target=$file";
-      $edit = "edit?dir=$dir&amp;file=$file";
-      $del = "javascript:del('$file');";
+      
+      return array(
+        "icon" => $icon,
+        "file" => $file,
+        "rev" => array($myrev['rev'],$rev),
+        "date" => $myrev['date'],
+        "author" => $myrev['author'],
+        "size" => $size
+      );
+      
+    }
+    else 
+    {
+      $this->info("Unknown RCS entry type : '$myitem'");
+      return;
     }
 
-    $item = $click ? array("$file/",$click) : $file;
-    $actions = array();
-    if ($view) array_push($actions, array(__("view"),$view));
-    if ($edit && $canedit) array_push($actions, array(__("edit"),$edit));
-    if ($rev) array_push($actions, array(__("revisions"),$rev));
-    if ($del && $canedit) array_push($actions, array(__("delete"),$del));
-    return array(($odd ? "odd" : "even"),$icon, $item,array($myrev['rev'],$rev),$myrev['date'],$myrev['author'],$size,$actions);
-  }
-
-
-  /** Display an RCS entry's revisions.
-  *
-  * @param dir parent directory
-  * @param loc visible location for parent directory
-  * @param file the RCS entry
-  * @param canedit are we allowed to edit this entry?
-  */
-  function dispRevs($dir,$loc,$file,$canedit)
-  {
-    global $globals;
-
-    $entries = array();
-    $odd = false;
-
-    // build url
-    $url = ($loc ? "$loc/" : "") . $file;
-    $url = $this->caller->urlSite($url);
-    $urlPage = $this->caller->urlSite($loc ? "$loc/" : "");
-
-    // process log entries
-    $revs = $this->logParse($dir,$file);
-    $head = $revs[0]['rev'];
-
-    $sz = count($revs);
-    for ($i = 0; $i < $sz; $i++) {
-      $rev = $revs[$i];
-      $actions = array(array(__("view"),"$url?rev={$rev['rev']}"));
-      if ($file == $globals->htmlfile) {
-        array_push($actions, array(__("view page"),"$urlPage?rev={$rev['rev']}"));
-      }
-      if ($i != ($sz-1)) {
-        array_push($actions, array(__("diff to")." {$revs[$i+1]['rev']}",
-          "?action=diff&amp;dir=$dir&amp;target=$file&amp;r1={$revs[$i+1]['rev']}&amp;r2={$rev['rev']}&amp;author={$rev['author']}"));
-      }
-      if (($i != 0) && $canedit) {
-        array_push($actions, array(__("restore"),
-          "javascript:restore('{$rev['rev']}');"));
-      }
-      array_push($entries, array(($odd ? 'odd' : 'even'),$rev['rev'],$rev['date'],
-                 $rev['author'],$rev['log'], $actions));
-    }
-
-    return $entries;
   }
 
 
